@@ -3,10 +3,33 @@
 package KiokuDB::Backend::Serialize;
 use Moose::Role;
 
+use Moose::Util::TypeConstraints;
+
 use namespace::clean -except => 'meta';
 
 requires qw(serialize deserialize);
 
+my %types = (
+    storable => "KiokuDB::Serializer::Storable",
+    json     => "KiokuDB::Serializer::JSON",
+    yaml     => "KiokuDB::Serializer::YAML",
+    memory   => "KiokuDB::Serializer::Memory",
+);
+
+coerce( __PACKAGE__,
+    from Str => via {
+        my $class = $types{lc($_)};
+        Class::MOP::load_class($class);
+        $class->new;
+    },
+    from HashRef => via {
+        my %args = %$_;
+        my $class = $types{lc(delete $args{format})};
+        Class::MOP::load_class($class);
+        $class->new(%args);
+    },
+);
+
 __PACKAGE__
 
 __END__
