@@ -67,6 +67,12 @@ sub precheck {
 
     my $backend = $self->backend;
 
+    if ( $backend->does("KiokuDB::Backend::Role::Broken") ) {
+        foreach my $fixture ( $backend->skip_fixtures ) {
+            $self->skip_fixture("broken backend") if $fixture eq ref($self) or $fixture eq $self->name;
+        }
+    }
+
     my @missing;
 
     role: foreach my $role ( $self->required_backend_roles ) {
@@ -231,15 +237,15 @@ sub lookup_obj_ok {
     return $obj;
 }
 
-sub no_live_objects {
-    my $self = shift;
+sub _no_live_objects {
+    my ( $self, $entries ) = @_;
 
     local $Test::Builder::Level = $Test::Builder::Level + 1;
 
     my $fail;
 
     $fail++ unless is( scalar(()=$self->live_objects), 0, "no live objects" );
-    #$fail++ unless is( scalar($self->directory->live_objects->live_entries), 0, "no live entries" );
+    $fail++ if $entries && not is( scalar($self->directory->live_objects->live_entries), 0, "no live entries" );
 
     if ( $fail ) {
         my @l = $self->live_objects;
@@ -260,6 +266,16 @@ sub no_live_objects {
     }
 }
 
+sub no_live_objects {
+    my $self = shift;
+    $self->_no_live_objects(0);
+}
+
+sub no_live_entries {
+    my $self = shift;
+    $self->_no_live_objects(1);
+}
+
 sub live_objects_are {
     my ( $self, @objects ) = @_;
 
